import * as tough from 'tough-cookie';
import { Request, IGBody, IGLogin, IGResponse, IGUserFollowersBody, IGUserFollowsBody, IGUserPostsBody, IGUserResponseObject, IGOpenPostBody, IGFollowerNode, IGUserPostNode, IGLocationBody } from './RequestWeb';
import { GenericStream, CustomSuccessHandler } from './GenericStream';
export interface SDKOptions {
    cookieJarStore?: tough.Store;
    repeatOnTooManyRequestsInterval?: number;
}
export declare type getCookies = (currentUrl: string, options?: tough.CookieJar.GetCookiesOptions) => Promise<tough.Cookie[]>;
export declare type getCookieString = (currentUrl: string, options?: tough.CookieJar.GetCookiesOptions) => Promise<string>;
export declare type setCookie = (cookieOrString: tough.Cookie | string, currentUrl: string, options?: tough.CookieJar.SetCookieOptions) => Promise<tough.Cookie[]>;
export declare class IGSDK {
    static Errors: {
        LoginError: any;
        ThrottledError: any;
    };
    static alphabet: string;
    static idToShortCode(id: number | string): string;
    static shortCodeToId(code: string): string;
    static withCredentials(username: string, password: string, options?: SDKOptions): IGSDK;
    static genHeaders(token: string, cookies: tough.Cookie[]): {
        referer: string;
        'x-csrftoken': string;
    };
    options: SDKOptions;
    username: string;
    password: string;
    jar: tough.CookieJar;
    logLevel: string;
    token: string;
    getCookies: getCookies;
    getCookieString: getCookieString;
    setCookie: setCookie;
    private defaultOptions;
    constructor(options?: SDKOptions);
    extractCSRFToken<TBody>(res: IGResponse<TBody>): Promise<void>;
    makeStream<TQuery, TBody, TOut>(method: string, pathToItems: string, pathToPageInfo: string, paginationProp: string, query: TQuery, customSuccessHandler?: CustomSuccessHandler<TQuery, TBody, TOut>): GenericStream<TQuery, TBody & IGBody, TOut>;
    getUserByName(userName: string): Promise<IGUserResponseObject>;
    login(force?: boolean): Promise<true | IGLogin>;
    isLoggedIn(): Promise<boolean>;
    getPostById(id: number | string): Promise<IGOpenPostBody>;
    getPostByShortCode(shortCode: string): Promise<IGOpenPostBody>;
    getLocationById(locationId: string | number): Promise<IGLocationBody>;
    graphQLQuery<T, TBody>(opts: T, queryId: string): Promise<IGResponse<TBody> | TBody>;
    getUserFollowers(opts: GenericGraphQLOptions): Promise<IGUserFollowersBody | IGResponse<IGUserFollowersBody>>;
    getUserFollows(opts: GenericGraphQLOptions): Promise<IGUserFollowsBody | IGResponse<IGUserFollowsBody>>;
    getUserPosts(opts: GenericGraphQLOptions): Promise<IGUserPostsBody | IGResponse<IGUserPostsBody>>;
    makeUserFollowersSteam(opts: GenericGraphQLOptions): GenericStream<GenericGraphQLOptions, IGUserFollowersBody & IGBody, IGFollowerNode>;
    makeUserFollowsSteam(opts: GenericGraphQLOptions): GenericStream<GenericGraphQLOptions, IGUserFollowsBody & IGBody, IGFollowerNode>;
    makeUserPostsSteam(opts: GenericGraphQLOptions): GenericStream<GenericGraphQLOptions, IGUserPostsBody & IGBody, IGUserPostNode>;
    request<TBody>(path: string): Request<TBody>;
}
export interface GenericGraphQLOptions {
    id: number;
    first: number;
    after?: string;
}
export { IGBody, IGLogin, IGResponse, IGFollowerNode, IGUserFollowersBody, IGUserFollowsBody, IGUserPostsBody, IGUserResponseObject, IGUserObject, IGUserMedia, Paginable, IGUserPostNode, IGUserAllMediaNode, IGTextNode, IGUserMediaGenericNode, IGUserLastMediaNode, IGUserMediaThumbnail, IGCountable, IGPostOwnerObject, IGUserMinimalObject, IGPostLikeNode, IGPostCommentNode, IGPostTaggedNode, IGPostOpenObject, IGOpenPostBody, IGUserMediaGenericListNode, IGLocationBody, IGLocationObject } from './RequestWeb';
export { GenericStream } from './GenericStream';
