"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const _ = require("lodash");
const tough = require("tough-cookie");
const got = require("got");
const tunnel = require("tunnel");
const Constants_1 = require("./Constants");
const Signatures_1 = require("./Signatures");
const Errors_1 = require("./Errors");
const logger_1 = require("./lib/logger");
const Cookie = tough.Cookie;
class Request {
    constructor(ig, url) {
        this.ig = ig;
        this.requestId = Signatures_1.default.generateUUID();
        this.method = 'GET';
        this.body = {};
        this.query = {};
        this.headers = {
            Connection: 'keep-alive',
            Accept: '*/*',
            'Accept-Encoding': Constants_1.default.ACCEPT_ENCODING,
            'X-IG-Capabilities': Constants_1.default.X_IG_Capabilities,
            'X-IG-Connection-Type': Constants_1.default.X_IG_Connection_Type,
            'X-IG-Connection-Speed': _.random(1000, 3700) + 'kbps',
            'X-FB-HTTP-Engine': Constants_1.default.X_FB_HTTP_Engine,
            'Content-Type': Constants_1.default.CONTENT_TYPE,
            'Accept-Language': Constants_1.default.ACCEPT_LANGUAGE,
        };
        this.defaultRequestOptions = {
            timeout: 5000,
        };
        this.defaultRequestOptions2 = {
            decompress: true,
            followRedirect: true,
        };
        this.checkStatusCode = false;
        this.successStatusCodes = new Set([200, 204]);
        this.repeatOnTooManyRequestsInterval = 0;
        this.parseResponseInJSON = true;
        this.reloginOnError = true;
        this.url = Constants_1.default.API_URL + url;
        this.logger = logger_1.default.getLogger('instagram-sdk:Request', { level: this.ig.logLevel });
        this.headers['User-Agent'] = this.ig.userAgent;
        // this.defaultRequestOptions.jar = this.ig.jar;
        // this.defaultRequestOptions.baseUrl = Constants.API_URL;
    }
    static _normalizeQueryParams(query) {
        const out = {};
        for (const i in query) {
            if (Object.prototype.hasOwnProperty.call(query, i)) {
                if (query[i] !== undefined && query[i] !== null) {
                    out[i] = query[i];
                }
            }
        }
        return out;
    }
    setReloginOnError(relogin) {
        this.reloginOnError = relogin;
        return this;
    }
    setParseResponseInJSON(parse) {
        this.parseResponseInJSON = parse;
        return this;
    }
    setSuccessStatusCodes(codes) {
        this.successStatusCodes = new Set(codes);
        return this;
    }
    setCheckStatusCode(check) {
        this.checkStatusCode = check;
        return this;
    }
    setMethod(method) {
        this.method = method;
        return this;
    }
    setBody(body) {
        this.body = body;
        return this;
    }
    setQuery(query) {
        this.query = query;
        return this;
    }
    setHeaders(headers) {
        this.headers = headers;
        return this;
    }
    addBodyParam(key, value) {
        this.body[key] = value;
        return this;
    }
    addQueryParam(key, value) {
        this.query[key] = value;
        return this;
    }
    addHeader(key, value) {
        this.headers[key] = value;
        return this;
    }
    then(...args) {
        this.cachedSendPromise = this.cachedSendPromise || this.send();
        return this.cachedSendPromise.then(...arguments);
    }
    catch(...args) {
        this.cachedSendPromise = this.cachedSendPromise || this.send();
        return this.cachedSendPromise.catch(...arguments);
    }
    send() {
        return __awaiter(this, void 0, void 0, function* () {
            const options = Object.assign({}, this.defaultRequestOptions2, {
                method: this.method,
                headers: Object.assign({}, this.headers, {
                    cookie: yield util.promisify(this.ig.jar.getCookieString.bind(this.ig.jar))(this.url),
                }),
                json: true,
                query: Request._normalizeQueryParams(this.query),
            });
            if (this.ig.proxy) {
                options.agent = tunnel.httpOverHttp({
                    proxy: {
                        host: this.ig.proxy.ip,
                        port: this.ig.proxy.port,
                        headers: {},
                    },
                });
                this.logger.debug(`(${this.requestId}) Request.send: using proxy:`, this.ig.proxy);
            }
            if (!_.isEmpty(this.body)) {
                options.body = /*JSON.stringify(*/ this.body /*)*/;
                options.form = true;
                this.logger.debug(`(${this.requestId}) Request.send: request with body: %:2j`, this.body);
            }
            this.logger.debug(`(${this.requestId}) Request.send: request to %s with params: %:2j`, this.url, options);
            return got(this.url, options).catch((err) => {
                if (err instanceof got.HTTPError) {
                    if (this.checkStatusCode && !this.successStatusCodes.has(err.statusCode)) {
                        if (err.statusCode === 429) {
                            const error = new Errors_1.default.ThrottledError();
                            error.res = err.response;
                            throw error;
                        }
                    }
                    else {
                        this.logger.debug(err);
                        return Promise.resolve(err.response);
                    }
                }
                err.res = { requestId: this.requestId };
                throw err;
            }).then((res) => __awaiter(this, void 0, void 0, function* () {
                res.requestId = this.requestId;
                this.logger.debug(`(${this.requestId}) Request.send: response statusCode:`, res.statusCode);
                this.logger.debug(`(${this.requestId}) Request.send: response headers:`, res.headers);
                let cookies;
                if (Array.isArray(res.headers['set-cookie'])) {
                    cookies = res.headers['set-cookie'].map((str) => Cookie.parse(str));
                }
                else {
                    cookies = [Cookie.parse(res.headers['set-cookie'])];
                }
                const setCookie = util.promisify(this.ig.jar.setCookie.bind(this.ig.jar));
                for (const cookie of cookies) {
                    yield setCookie(cookie, this.url);
                }
                yield this.ig.extractCSRFToken({ res });
                if (this.repeatOnTooManyRequestsInterval && res.statusCode === 429) {
                    return setTimeout(this.send.bind(this), this.repeatOnTooManyRequestsInterval);
                }
                if (this.reloginOnError && res.body.message === 'login_required') {
                    this.logger.debug(`(${this.requestId}) Request.send: 'login_required', try again:`, res.body);
                    return this.ig.login(true).then(() => {
                        return this.send();
                    });
                }
                this.logger.debug(`(${this.requestId}) Request.send parsing JSON: response: %.-500s`, util.inspect(res.body));
                if (res.body) {
                    res.body.requestId = this.requestId;
                }
                return Promise.resolve(res.body);
            }));
        });
    }
}
exports.default = Request;
