/// <reference types="node" />
/// <reference types="got" />
/// <reference types="winston" />
import * as http from 'http';
import * as tough from 'tough-cookie';
import * as got from 'got';
import { LoggerInstance } from 'winston';
import { IGSDK } from './InstagramSDKWeb';
export declare class Request<TBody> {
    sdk: IGSDK;
    static baseUrl: string;
    static parseCookies(headers: http.IncomingHttpHeaders): tough.Cookie[];
    private static _normalizeQueryParams(query);
    requestId: string;
    method: string;
    body: {};
    query: {};
    headers: http.OutgoingHttpHeaders;
    defaultRequestOptions: got.GotOptions<string | null>;
    checkStatusCode: boolean;
    successStatusCodes: Set<number>;
    parseResponseInJSON: boolean;
    reloginOnError: boolean;
    logger: LoggerInstance;
    url: string;
    response: null | IGResponse<TBody>;
    constructor(sdk: IGSDK, url: string);
    setReloginOnError(relogin: boolean): this;
    setParseResponseInJSON(parse: boolean): this;
    setSuccessStatusCodes(codes: number[]): this;
    setCheckStatusCode(check: boolean): this;
    setMethod(method: string): this;
    setBody(body: object): this;
    setQuery(query: object): this;
    setHeaders(headers: http.OutgoingHttpHeaders): this;
    getResultHeaders(): Promise<http.OutgoingHttpHeaders | undefined>;
    getResultQuery(): QueryString;
    getRequestOptions(headers: http.OutgoingHttpHeaders | undefined, query: QueryString): got.GotJSONOptions;
    addBodyParam(key: string, value: string): this;
    addQueryParam(key: string, value: string): this;
    addHeader(key: string, value: string): this;
    send(): Promise<TBody>;
}
export interface IGResponse<T> extends got.Response<T & IGBody> {
    requestId?: string;
}
export interface IGBody {
    requestId: string;
    status?: string;
    message?: string;
    data?: any;
}
export declare type QueryString = object | undefined;
export interface IGLogin extends IGBody {
    authenticated: boolean;
    user: boolean;
    reactivated: boolean;
    status: 'ok';
}
export interface IGFollowerNode {
    node: IGFollowerObject;
}
export interface IGFollowerObject extends IGUserMinimalObject {
    full_name: string | null;
    is_verified: boolean;
}
export interface IGUserFollowersBody extends IGBody {
    data: {
        user: {
            edge_followed_by: Paginable & {
                edges: IGFollowerNode[];
            };
        };
    };
}
export interface IGUserFollowsBody extends IGBody {
    data: {
        user: {
            edge_follow: Paginable & {
                edges: IGFollowerNode[];
            };
        };
    };
}
export interface IGUserPostsBody extends IGBody {
    data: {
        user: {
            edge_owner_to_timeline_media: Paginable & {
                edges: IGUserPostNode[];
            };
        };
    };
}
export interface IGUserResponseObject extends IGBody {
    logging_page_id?: string;
    user: IGUserObject;
}
export interface IGUserObject {
    biography: string | null;
    external_url: string | null;
    external_url_linkshimmed: string | null;
    followed_by: IGCountable;
    follows: IGCountable;
    full_name: string | null;
    id: string;
    is_private: boolean;
    is_verified: boolean;
    profile_pic_url: string;
    profile_pic_url_hd: string;
    username: string;
    media: IGUserMedia;
    saved_media: IGUserMedia;
}
export interface IGUserMedia extends Paginable {
    nodes: IGUserLastMediaNode[];
}
export interface Paginable {
    count: number;
    page_info: {
        has_next_page: boolean;
        end_cursor: string | null;
    };
}
export interface IGUserPostNode {
    node: IGUserAllMediaNode;
}
export interface IGTextNode {
    node: {
        text: string;
    };
}
export interface IGUserMediaGenericListNode extends IGUserMediaGenericNode {
    owner: {
        id: string;
    };
    thumbnail_src: string;
    thumbnail_resources: IGUserMediaThumbnail[];
}
export interface IGUserMediaGenericNode {
    __typename?: 'GraphImage' | 'GraphVideo' | 'GraphSidecar';
    id: string;
    comments_disabled?: boolean;
    dimensions: {
        width: number;
        height: number;
    };
    is_video: boolean;
}
export interface IGUserAllMediaNode extends IGUserMediaGenericListNode {
    edge_media_to_caption: {
        edges: IGTextNode[];
    };
    shortcode: string;
    edge_media_to_comment: IGCountable;
    taken_at_timestamp: number;
    display_url: string;
    edge_media_preview_like: IGCountable;
}
export interface IGUserLastMediaNode extends IGUserMediaGenericListNode {
    gating_info?: null;
    media_preview?: string | null;
    code: string;
    date: number;
    display_src: string;
    video_views?: number;
    caption: string;
    comments: IGCountable;
    likes: IGCountable;
}
export interface IGUserMediaThumbnail {
    src: string;
    config_width: number;
    config_height: number;
}
export interface IGCountable {
    count: number;
}
export interface IGOpenPostBody {
    graphql: {
        shortcode_media: IGPostOpenObject;
    };
}
export interface IGPostOpenObject extends IGUserMediaGenericNode {
    shortcode: string;
    gating_info: null;
    media_preview: string | null;
    display_url: string;
    display_resources: IGUserMediaThumbnail[];
    video_url?: string;
    video_view_count?: number;
    edge_media_to_tagged_user: {
        edges: IGPostTaggedNode[];
    };
    edge_media_to_caption: {
        edges: IGTextNode[];
    };
    edge_media_to_comment: Paginable & {
        edges: IGPostCommentNode[];
    };
    taken_at_timestamp: number;
    edge_media_preview_like: IGCountable & {
        edges: IGPostLikeNode[];
    };
    location: IGPostOpenLocationObject | null;
    owner: IGPostOwnerObject;
    is_ad: boolean;
}
export interface IGPostOpenLocationObject {
    id: string;
    has_public_page: boolean;
    name: string;
    slug: string;
}
export interface IGPostTaggedNode {
    node: {
        user: {
            username: string;
        };
        x: number;
        y: number;
    };
}
export interface IGPostCommentNode {
    node: {
        id: string;
        text: string;
        created_at: number;
        owner: IGUserMinimalObject;
    };
}
export interface IGPostLikeNode {
    node: IGUserMinimalObject;
}
export interface IGUserMinimalObject {
    id: string;
    profile_pic_url: string;
    username: string;
}
export interface IGPostOwnerObject extends IGUserMinimalObject {
    full_name: string | null;
    is_private: boolean;
    is_verified: boolean;
}
export interface IGLocationBody {
    location: IGLocationObject;
    logging_page_id: string;
}
export interface IGLocationObject {
    id: string;
    name: string;
    has_public_page: boolean;
    lat: number;
    lng: number;
    slug: string;
    media: IGLocationMedia;
    top_posts: IGLocationMedia;
    directory: Directory;
}
export interface IGLocationMedia extends Paginable {
    nodes: IGLocationGenericMediaNode[];
}
export interface IGLocationGenericMediaNode extends IGUserLastMediaNode {
    edge_media_preview_like: IGCountable;
}
export interface Directory {
    country: CountryOrCity;
    city: CountryOrCity;
}
export interface CountryOrCity {
    id: string;
    name: string;
    slug: string;
}
