"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
class UserSettings {
    constructor(baseDir, username, redisClient) {
        this.baseDir = baseDir;
        this.username = username;
        this.redisClient = redisClient;
        this.prefix = 'big-brother:instagram:settings:';
        if (!redisClient) {
            throw new Error('`redisClient` is required.');
        }
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.redisClient.get(this.prefix + key);
        });
    }
    set(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.redisClient.set(this.prefix + key, value);
        });
    }
    loadUserSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = yield this.getAllKeys();
            return lodash_1.fromPairs(keys.map((key) => {
                return [key.split(':').slice(-1)[0], this.redisClient.get(key)];
            }));
        });
    }
    isMaybeLoggedIn() {
        return __awaiter(this, void 0, void 0, function* () {
            let hasKeys;
            try {
                hasKeys = !!(yield this.getAllKeys()).length;
            }
            catch (err) {
                hasKeys = false;
            }
            return hasKeys && (yield this.get('account_id')) && (yield this.get('token'));
        });
    }
    getAllKeys() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.redisClient.keys(this.prefix + '*');
        });
    }
}
exports.default = UserSettings;
;
